@@ -27,11 +27,15 @@ fn _rust_backend(_py: Python, m: &PyModule) -> PyResult<()> {
     m.add_function(wrap_pyfunction!(linalg::solve_ols, m)?)?;
     m.add_function(wrap_pyfunction!(linalg::compute_robust_vcov, m)?)?;
 
-    // TROP estimator acceleration
+    // TROP estimator acceleration (twostep method)
     m.add_function(wrap_pyfunction!(trop::compute_unit_distance_matrix, m)?)?;
     m.add_function(wrap_pyfunction!(trop::loocv_grid_search, m)?)?;
     m.add_function(wrap_pyfunction!(trop::bootstrap_trop_variance, m)?)?;
 
+    // TROP estimator acceleration (joint method)
+    m.add_function(wrap_pyfunction!(trop::loocv_grid_search_joint, m)?)?;
+    m.add_function(wrap_pyfunction!(trop::bootstrap_trop_variance_joint, m)?)?;
+
     // Version info
     m.add("__version__", env!("CARGO_PKG_VERSION"))?;
 
